﻿Option Strict Off
Option Explicit On

Imports System.Windows.Forms

#Const DataPump = False   ' True -> DataPump  False -> TradePump   ' *** Need to change assembly name to/from "FXCM Data Server" and "FXCM Forex Broker" in My Project

Module NSTOrdersAPI
#If Not DataPump Then
    ' This moudle contains procedures that can be called to simplify coding the interface to the NeuroShell Trader Trade Pump dll

    ' *****************************************************************************************************
    ' SETTINGS
    ' *****************************************************************************************************
    Public Const FillPriceFeedback As Short = 1
    Public Const DistinctShortOrders As Short = 1
    Public Const AllowOrderModification As Short = 0

    ' *********************************************************
    ' NeuroShell Trader Order Constants
    ' *********************************************************

    Public Const MARKET_ORDER As Integer = 1
    Public Const STOP_ORDER As Integer = 2
    Public Const LIMIT_ORDER As Integer = 3
    Public Const STOPLIMIT_ORDER As Integer = 4
    Public Const MARKETCLOSE_ORDER As Integer = 5

    Public Const LONGENTRY_ACTION As Integer = 1
    Public Const SHORTENTRY_ACTION As Integer = 2
    Public Const LONGEXIT_ACTION As Integer = 3
    Public Const SHORTEXIT_ACTION As Integer = 4

    ' *****************************************************************************************************
    ' Trade Pump DLL Declares
    ' *****************************************************************************************************
    Private Declare Sub SetEXEOrderPlaceHWND Lib "NSTBkTrd.dll" (ByVal hwnd As Integer)
    Private Declare Sub SetBrokerageDefaults Lib "NSTBkTrd.dll" (ByVal FillPriceFeedback As Integer, ByVal DistinctShortOrders As Integer, ByVal AllowOrderModification As Integer)
    Private Declare Function OrderFilled Lib "NSTBkTrd.dll" (ByVal NSTOrderID As String, ByVal Shares As Integer, ByVal FillPrice As Double) As Integer
    Private Declare Function OrderCanceled Lib "NSTBkTrd.dll" (ByVal NSTOrderID As String) As Integer
    Private Declare Function OrderInactive Lib "NSTBkTrd.dll" (ByVal NSTOrderID As String) As Integer
    Private Declare Function OrderIdChange Lib "NSTBkTrd.dll" (ByVal OldOrderID As String, ByVal NewOrderID As String) As Integer
    'Not implemented in DLL at this time -> Private Declare Function OrderStatusUpdate Lib "NSTBkTrd.dll" (ByVal NSTOrderID As String, ByVal FilledShares As Integer, ByVal AvgFillPrice As Double, ByVal FillDateTime As Double) As Integer
    Private Declare Function VerifyTickerResult Lib "NSTBkTrd.dll" Alias "EXEIsValidSymbolSet" (ByVal Valid As Integer) As Integer
    Private Declare Function LogonResult Lib "NSTBkTrd.dll" Alias "EXEBrokerageLogonSet" (ByVal Valid As Integer) As Integer
    Public Declare Function Message Lib "NSTBkTrd.dll" Alias "Message" (ByVal Text As String) As Integer
    Private Declare Function Disconnected Lib "NSTBkTrd.dll" Alias "Disconnected" () As Integer
    Private Declare Function Reconnected Lib "NSTBkTrd.dll" Alias "Reconnected" () As Integer

    Private Declare Function DLLLogonResult Lib "NSTBkTrd.dll" Alias "EXEBrokerageLogonSet" (ByVal Valid As Long) As Long
    Private Declare Function DLLAccountBalanceResult Lib "NSTBkTrd.dll" Alias "EXEBrokerageAccountBalanceSet" (ByVal Valid As Long) As Long
    Private Declare Function DLLOrderIdChange Lib "NSTBkTrd.dll" Alias "OrderIdChange" (ByVal OldOrderID As String, ByVal NewOrderID As String) As Long
    Private Declare Function DLLVerifyTickerResult Lib "NSTBkTrd.dll" Alias "EXEIsValidSymbolSet" (ByVal Valid As Long) As Long

    Public OrderEventsTextBox As TextBox      ' a dynamically created text box to receive trade pump events

    ' *****************************************************************************************************
    ' Public Subs
    ' *****************************************************************************************************

    Public Sub TradePump_Connect() 'ByVal TextHwnd As Integer)
        LogFile("TradePump_Connect")

        ' ********************************************
        ' MUST BE CALLED WHEN APPLICATION FIRST STARTS
        ' ********************************************

        ' dynamically create the text box that will receive data pump events
        OrderEventsTextBox = New TextBox
        AddHandler OrderEventsTextBox.TextChanged, AddressOf OrderEventsTextBox_TextChanged

        ' Send NST Trade Pump the handle to the text box that trade pump will use to send events - 
        SetEXEOrderPlaceHWND(OrderEventsTextBox.Handle.ToInt32)

        ' Let NST Trade Pump know the values of FillPriceFeedback, DistinctShortOrders and AllowOrderModification
        SetBrokerageDefaults(FillPriceFeedback, DistinctShortOrders, AllowOrderModification)
    End Sub

    Public Sub TradePump_Disconnect()
        LogFile("TradePump_Disconnect")

        ' ********************************************
        ' MUST BE CALLED BEFORE APPLICATION CLOSES
        ' ********************************************

        RemoveHandler OrderEventsTextBox.TextChanged, AddressOf OrderEventsTextBox_TextChanged

        ' Clear the handle to the text box that data pump will use to send events, so the NST Data Pump doesn't keep trying to send events
        SetEXEOrderPlaceHWND(0)
    End Sub

    Public Sub TradePump_SetOrderId(ByVal OldOrderID As String, ByVal NewOrderID As String)
        LogFile("TradePump_SetOrderId(" + OldOrderID + ", " + NewOrderID + ")")

        ' ********************************************
        ' Sets OrderId used by NST to the orderid used by the brokerage (so that future NST modify, cancel, etc. requests will use the brokerage order id instead of the original NST order id)
        ' ********************************************

        Dim ret As Integer = OrderIdChange(OldOrderID, NewOrderID)
    End Sub

    Public Sub TradePump_OrderFilled(ByVal NSTOrderId As String, ByVal Shares As Integer, ByVal FillPrice As Double)
        LogFile("TradePump_OrderFilled(" + NSTOrderId + ", " + CStr(Shares) + ", " + CStr(FillPrice) + ")")

        ' ********************************************
        ' Sends order fill information back to Trade Pump (only used if using FillPriceFeedback)
        ' ********************************************

        If FillPriceFeedback <> 0 Then
            Dim ret As Integer = OrderFilled(NSTOrderId, Shares, FillPrice)
            LogFile("NSTOrderFilled finished for " + NSTOrderId + " Shares=" + CStr(Shares) + " Price=" + CStr(FillPrice))
        End If
    End Sub

    Public Sub TradePump_OrderCanceled(ByVal NSTOrderId As String)
        LogFile("TradePump_OrderCanceled(" + NSTOrderId + ")")

        ' ********************************************
        ' Sends order cancel confirmation back to Trade Pump (only used if using FillPriceFeedback)
        ' ********************************************

        If FillPriceFeedback <> 0 Then
            Dim ret As Integer = OrderCanceled(NSTOrderId)
            LogFile("NSTOrderCanceled sent for " + NSTOrderId)
        End If
    End Sub


    ' *****************************************************************************************************
    ' Private Subs
    ' *****************************************************************************************************

    Public Sub OrderEventsTextBox_TextChanged(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs)
        Dim col As New Collection
        Dim OrderId As String
        Dim ret As Long
        Dim txt As String

        txt = OrderEventsTextBox.Text
        LogFile("OrderEvents(" + txt + ")")

        Do While InStr(txt, "!@#") > 0
            col.Add(Trim(Left(txt, InStr(txt, "!@#") - 1)))
            txt = Right(txt, Len(txt) - InStr(txt, "!@#") - 2)
        Loop
        col.Add(txt)

        Select Case col(1)
            Case "Unload"
                UnloadOrders()
            Case "LogOn"
                ' fire logon event
                ret = True  ' assume logged on unless changed in event
                LogonOrders(col(2), col(3), ret)

                ' send back logon result
                DLLLogonResult(ret)
            Case "AccountBalance"
                ' send back logon result
                DLLAccountBalanceResult(AccountBalance())
            Case "LogOff"
                ' fire loggoff event
                LogoffOrders()
            Case "OrderStatus"
                ' fire cancel order event
                'RaiseEvent CancelOrder(col(2))
            Case "CancelOrder"
                ' fire cancel order event
                CancelOrder(col(2))
            Case "VerifyTicker"
                ' fire cancel order event
                ret = 1  ' assume valid unless changed in event
                VerifyTicker(col(2), col(3), col(4), ret)

                ' send back verify ticker result
                DLLVerifyTickerResult(ret)

            Case "ModifyOrder"
                ' fire modify order event
                ModifyOrder(CStr(col(2)), CDbl(col(3)), CDbl(col(4)))
                MsgBox("Modify Order id " + CStr(col(2)) + " fired " + CStr(col(3)) + " / " + CStr(col(4)))
            Case "NewOrder"
                ' fire new order event
                OrderId = CStr(col(2))
                NewOrder(OrderId, CLng(col(3)), CLng(col(4)), CStr(col(5)), CLng(col(6)), CDbl(col(7)), CDbl(col(8)), CStr(col(9)), CStr(col(10)), CStr(col(11)), CStr(col(12)), CStr(col(13)), CStr(col(14)))

                ' send back orderid (in case changed to brokerage orderid instead of NST order id)
                If CStr(col(2)) <> OrderId Then ret = DLLOrderIdChange(CStr(col(2)), OrderId)
                'Case "CheckOrderStatuses":
                '    ' fire check open orders event
                '    RaiseEvent CheckOrderStatuses(col(2))
                'Case "NewTick":
                '    ProcessTick col(2), col(3)
            Case Else
                MsgBox("Unrecognized Event '" + col(1) + "' in NSTOrdersAPI.OCX")
        End Select
    End Sub
#End If
End Module
